@@ -69,6 +69,81 @@ func TestRouteAddDel(t *testing.T) {
 
 }
 
+func TestRoute6AddDel(t *testing.T) {
+	tearDown := setUpNetlinkTest(t)
+	defer tearDown()
+
+	// create dummy interface
+	// IPv6 route added to loopback interface will be unreachable
+	la := NewLinkAttrs()
+	la.Name = "dummy_route6"
+	la.TxQLen = 1500
+	dummy := &Dummy{LinkAttrs: la}
+	if err := LinkAdd(dummy); err != nil {
+		t.Fatal(err)
+	}
+
+	// get dummy interface
+	link, err := LinkByName("dummy_route6")
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	// bring the interface up
+	if err := LinkSetUp(link); err != nil {
+		t.Fatal(err)
+	}
+
+	// remember number of routes before adding
+	// typically one route (fe80::/64) will be created when dummy_route6 is created
+	routes, err := RouteList(link, FAMILY_V6)
+	if err != nil {
+		t.Fatal(err)
+	}
+	nroutes := len(routes)
+
+	// add a gateway route
+	dst := &net.IPNet{
+		IP:   net.ParseIP("2001:db8::0"),
+		Mask: net.CIDRMask(64, 128),
+	}
+	route := Route{LinkIndex: link.Attrs().Index, Dst: dst}
+	if err := RouteAdd(&route); err != nil {
+		t.Fatal(err)
+	}
+	routes, err = RouteList(link, FAMILY_V6)
+	if err != nil {
+		t.Fatal(err)
+	}
+	if len(routes) != nroutes+1 {
+		t.Fatal("Route not added properly")
+	}
+
+	dstIP := net.ParseIP("2001:db8::1")
+	routeToDstIP, err := RouteGet(dstIP)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	// cleanup route and dummy interface created for the test
+	if len(routeToDstIP) == 0 {
+		t.Fatal("Route not present")
+	}
+	if err := RouteDel(&route); err != nil {
+		t.Fatal(err)
+	}
+	routes, err = RouteList(link, FAMILY_V6)
+	if err != nil {
+		t.Fatal(err)
+	}
+	if len(routes) != nroutes {
+		t.Fatal("Route not removed properly")
+	}
+	if err := LinkDel(link); err != nil {
+		t.Fatal(err)
+	}
+}
+
 func TestRouteReplace(t *testing.T) {
 	tearDown := setUpNetlinkTest(t)
 	defer tearDown()
@@ -152,6 +227,7 @@ func TestRouteAddIncomplete(t *testing.T) {
 	}
 }
 
+// expectNeighUpdate returns whether the expected updated is received within one minute.
 func expectRouteUpdate(ch <-chan RouteUpdate, t uint16, dst net.IP) bool {
 	for {
 		timeout := time.After(time.Minute)
@@ -434,19 +510,21 @@ func TestRouteFilterAllTables(t *testing.T) {
 			Table:     table,
 			Type:      unix.RTN_UNICAST,
 			Tos:       14,
+			Hoplimit:  100,
 		}
 		if err := RouteAdd(&route); err != nil {
 			t.Fatal(err)
 		}
 	}
 	routes, err := RouteListFiltered(FAMILY_V4, &Route{
-		Dst:   dst,
-		Src:   src,
-		Scope: unix.RT_SCOPE_LINK,
-		Table: unix.RT_TABLE_UNSPEC,
-		Type:  unix.RTN_UNICAST,
-		Tos:   14,
-	}, RT_FILTER_DST|RT_FILTER_SRC|RT_FILTER_SCOPE|RT_FILTER_TABLE|RT_FILTER_TYPE|RT_FILTER_TOS)
+		Dst:      dst,
+		Src:      src,
+		Scope:    unix.RT_SCOPE_LINK,
+		Table:    unix.RT_TABLE_UNSPEC,
+		Type:     unix.RTN_UNICAST,
+		Tos:      14,
+		Hoplimit: 100,
+	}, RT_FILTER_DST|RT_FILTER_SRC|RT_FILTER_SCOPE|RT_FILTER_TABLE|RT_FILTER_TYPE|RT_FILTER_TOS|RT_FILTER_HOPLIMIT)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -470,6 +548,9 @@ func TestRouteFilterAllTables(t *testing.T) {
 		if route.Tos != 14 {
 			t.Fatal("Invalid Tos. Route not added properly")
 		}
+		if route.Hoplimit != 100 {
+			t.Fatal("Invalid Hoplimit. Route not added properly")
+		}
 	}
 }
 
@@ -512,18 +593,20 @@ func TestRouteExtraFields(t *testing.T) {
 		Table:     unix.RT_TABLE_MAIN,
 		Type:      unix.RTN_UNICAST,
 		Tos:       14,
+		Hoplimit:  100,
 	}
 	if err := RouteAdd(&route); err != nil {
 		t.Fatal(err)
 	}
 	routes, err := RouteListFiltered(FAMILY_V4, &Route{
-		Dst:   dst,
-		Src:   src,
-		Scope: unix.RT_SCOPE_LINK,
-		Table: unix.RT_TABLE_MAIN,
-		Type:  unix.RTN_UNICAST,
-		Tos:   14,
-	}, RT_FILTER_DST|RT_FILTER_SRC|RT_FILTER_SCOPE|RT_FILTER_TABLE|RT_FILTER_TYPE|RT_FILTER_TOS)
+		Dst:      dst,
+		Src:      src,
+		Scope:    unix.RT_SCOPE_LINK,
+		Table:    unix.RT_TABLE_MAIN,
+		Type:     unix.RTN_UNICAST,
+		Tos:      14,
+		Hoplimit: 100,
+	}, RT_FILTER_DST|RT_FILTER_SRC|RT_FILTER_SCOPE|RT_FILTER_TABLE|RT_FILTER_TYPE|RT_FILTER_TOS|RT_FILTER_HOPLIMIT)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -546,6 +629,9 @@ func TestRouteExtraFields(t *testing.T) {
 	if routes[0].Tos != 14 {
 		t.Fatal("Invalid Tos. Route not added properly")
 	}
+	if routes[0].Hoplimit != 100 {
+		t.Fatal("Invalid Hoplimit. Route not added properly")
+	}
 }
 
 func TestRouteMultiPath(t *testing.T) {
@@ -764,6 +850,12 @@ func TestRouteEqual(t *testing.T) {
 			Tos:       1,
 			Gw:        net.IPv4(1, 1, 1, 1),
 		},
+		{
+			LinkIndex: 20,
+			Dst:       nil,
+			Hoplimit:  1,
+			Gw:        net.IPv4(1, 1, 1, 1),
+		},
 		{
 			LinkIndex: 20,
 			Dst:       nil,
@@ -800,6 +892,19 @@ func TestRouteEqual(t *testing.T) {
 			Type:     unix.RTN_UNICAST,
 			Tos:      14,
 		},
+		{
+			LinkIndex: 3,
+			Dst: &net.IPNet{
+				IP:   net.IPv4(1, 1, 1, 1),
+				Mask: net.CIDRMask(32, 32),
+			},
+			Src:      net.IPv4(127, 3, 3, 3),
+			Scope:    unix.RT_SCOPE_LINK,
+			Priority: 13,
+			Table:    unix.RT_TABLE_MAIN,
+			Type:     unix.RTN_UNICAST,
+			Hoplimit: 100,
+		},
 		{
 			LinkIndex: 10,
 			MPLSDst:   &mplsDst,
@@ -917,9 +1022,108 @@ func TestIPNetEqual(t *testing.T) {
 	}
 }
 
+func TestSEG6LocalEqual(t *testing.T) {
+	// Different attributes exists in different Actions. For example, Action
+	// SEG6_LOCAL_ACTION_END_X has In6Addr, SEG6_LOCAL_ACTION_END_T has Table etc.
+	segs := []net.IP{net.ParseIP("fc00:a000::11")}
+	// set flags for each actions.
+	var flags_end [nl.SEG6_LOCAL_MAX]bool
+	flags_end[nl.SEG6_LOCAL_ACTION] = true
+	var flags_end_x [nl.SEG6_LOCAL_MAX]bool
+	flags_end_x[nl.SEG6_LOCAL_ACTION] = true
+	flags_end_x[nl.SEG6_LOCAL_NH6] = true
+	var flags_end_t [nl.SEG6_LOCAL_MAX]bool
+	flags_end_t[nl.SEG6_LOCAL_ACTION] = true
+	flags_end_t[nl.SEG6_LOCAL_TABLE] = true
+	var flags_end_dx2 [nl.SEG6_LOCAL_MAX]bool
+	flags_end_dx2[nl.SEG6_LOCAL_ACTION] = true
+	flags_end_dx2[nl.SEG6_LOCAL_OIF] = true
+	var flags_end_dx6 [nl.SEG6_LOCAL_MAX]bool
+	flags_end_dx6[nl.SEG6_LOCAL_ACTION] = true
+	flags_end_dx6[nl.SEG6_LOCAL_NH6] = true
+	var flags_end_dx4 [nl.SEG6_LOCAL_MAX]bool
+	flags_end_dx4[nl.SEG6_LOCAL_ACTION] = true
+	flags_end_dx4[nl.SEG6_LOCAL_NH4] = true
+	var flags_end_dt6 [nl.SEG6_LOCAL_MAX]bool
+	flags_end_dt6[nl.SEG6_LOCAL_ACTION] = true
+	flags_end_dt6[nl.SEG6_LOCAL_TABLE] = true
+	var flags_end_dt4 [nl.SEG6_LOCAL_MAX]bool
+	flags_end_dt4[nl.SEG6_LOCAL_ACTION] = true
+	flags_end_dt4[nl.SEG6_LOCAL_TABLE] = true
+	var flags_end_b6 [nl.SEG6_LOCAL_MAX]bool
+	flags_end_b6[nl.SEG6_LOCAL_ACTION] = true
+	flags_end_b6[nl.SEG6_LOCAL_SRH] = true
+	var flags_end_b6_encaps [nl.SEG6_LOCAL_MAX]bool
+	flags_end_b6_encaps[nl.SEG6_LOCAL_ACTION] = true
+	flags_end_b6_encaps[nl.SEG6_LOCAL_SRH] = true
+
+	cases := []SEG6LocalEncap{
+		{
+			Flags:  flags_end,
+			Action: nl.SEG6_LOCAL_ACTION_END,
+		},
+		{
+			Flags:   flags_end_x,
+			Action:  nl.SEG6_LOCAL_ACTION_END_X,
+			In6Addr: net.ParseIP("2001:db8::1"),
+		},
+		{
+			Flags:  flags_end_t,
+			Action: nl.SEG6_LOCAL_ACTION_END_T,
+			Table:  10,
+		},
+		{
+			Flags:  flags_end_dx2,
+			Action: nl.SEG6_LOCAL_ACTION_END_DX2,
+			Oif:    20,
+		},
+		{
+			Flags:   flags_end_dx6,
+			Action:  nl.SEG6_LOCAL_ACTION_END_DX6,
+			In6Addr: net.ParseIP("2001:db8::1"),
+		},
+		{
+			Flags:  flags_end_dx4,
+			Action: nl.SEG6_LOCAL_ACTION_END_DX4,
+			InAddr: net.IPv4(192, 168, 10, 10),
+		},
+		{
+			Flags:  flags_end_dt6,
+			Action: nl.SEG6_LOCAL_ACTION_END_DT6,
+			Table:  30,
+		},
+		{
+			Flags:  flags_end_dt4,
+			Action: nl.SEG6_LOCAL_ACTION_END_DT4,
+			Table:  40,
+		},
+		{
+			Flags:    flags_end_b6,
+			Action:   nl.SEG6_LOCAL_ACTION_END_B6,
+			Segments: segs,
+		},
+		{
+			Flags:    flags_end_b6_encaps,
+			Action:   nl.SEG6_LOCAL_ACTION_END_B6_ENCAPS,
+			Segments: segs,
+		},
+	}
+	for i1 := range cases {
+		for i2 := range cases {
+			got := cases[i1].Equal(&cases[i2])
+			expected := i1 == i2
+			if got != expected {
+				t.Errorf("Equal(%v,%v) == %s but expected %s",
+					cases[i1], cases[i2],
+					strconv.FormatBool(got),
+					strconv.FormatBool(expected))
+			}
+		}
+	}
+}
 func TestSEG6RouteAddDel(t *testing.T) {
-	// add/del IPv4 routes with LWTUNNEL_SEG6 to/from loopback interface.
-	// Test both seg6 modes: encap & inline.
+	// add/del routes with LWTUNNEL_SEG6 to/from loopback interface.
+	// Test both seg6 modes: encap (IPv4) & inline (IPv6).
 	tearDown := setUpSEG6NetlinkTest(t)
 	defer tearDown()
 
@@ -932,9 +1136,9 @@ func TestSEG6RouteAddDel(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	dst1 := &net.IPNet{
-		IP:   net.IPv4(10, 0, 0, 101),
-		Mask: net.CIDRMask(32, 32),
+	dst1 := &net.IPNet{ // INLINE mode must be IPv6 route
+		IP:   net.ParseIP("2001:db8::1"),
+		Mask: net.CIDRMask(128, 128),
 	}
 	dst2 := &net.IPNet{
 		IP:   net.IPv4(10, 0, 0, 102),
@@ -960,16 +1164,30 @@ func TestSEG6RouteAddDel(t *testing.T) {
 	if err := RouteAdd(&route2); err != nil {
 		t.Fatal(err)
 	}
-	routes, err := RouteList(link, FAMILY_V4)
+	// SEG6_IPTUN_MODE_INLINE
+	routes, err := RouteList(link, FAMILY_V6)
 	if err != nil {
 		t.Fatal(err)
 	}
-	if len(routes) != 2 {
+	if len(routes) != 1 {
 		t.Fatal("SEG6 routes not added properly")
 	}
 	for _, route := range routes {
 		if route.Encap.Type() != nl.LWTUNNEL_ENCAP_SEG6 {
-			t.Fatal("Invalid Type. SEG6 routes not added properly")
+			t.Fatal("Invalid Type. SEG6_IPTUN_MODE_INLINE routes not added properly")
+		}
+	}
+	// SEG6_IPTUN_MODE_ENCAP
+	routes, err = RouteList(link, FAMILY_V4)
+	if err != nil {
+		t.Fatal(err)
+	}
+	if len(routes) != 1 {
+		t.Fatal("SEG6 routes not added properly")
+	}
+	for _, route := range routes {
+		if route.Encap.Type() != nl.LWTUNNEL_ENCAP_SEG6 {
+			t.Fatal("Invalid Type. SEG6_IPTUN_MODE_ENCAP routes not added properly")
 		}
 	}
 
@@ -989,6 +1207,85 @@ func TestSEG6RouteAddDel(t *testing.T) {
 	}
 }
 
+// add/del routes with LWTUNNEL_ENCAP_SEG6_LOCAL to/from dummy interface.
+func TestSEG6LocalRoute6AddDel(t *testing.T) {
+	minKernelRequired(t, 4, 14)
+	tearDown := setUpSEG6NetlinkTest(t)
+	defer tearDown()
+
+	// create dummy interface
+	// IPv6 route added to loopback interface will be unreachable
+	la := NewLinkAttrs()
+	la.Name = "dummy_route6"
+	la.TxQLen = 1500
+	dummy := &Dummy{LinkAttrs: la}
+	if err := LinkAdd(dummy); err != nil {
+		t.Fatal(err)
+	}
+	// get dummy interface and bring it up
+	link, err := LinkByName("dummy_route6")
+	if err != nil {
+		t.Fatal(err)
+	}
+	if err := LinkSetUp(link); err != nil {
+		t.Fatal(err)
+	}
+
+	dst1 := &net.IPNet{
+		IP:   net.ParseIP("2001:db8::1"),
+		Mask: net.CIDRMask(128, 128),
+	}
+
+	// Create Route including Action SEG6_LOCAL_ACTION_END_B6.
+	// Could be any Action but thought better to have seg list.
+	var s1 []net.IP
+	s1 = append(s1, net.ParseIP("fc00:a000::12"))
+	s1 = append(s1, net.ParseIP("fc00:a000::11"))
+	var flags_end_b6_encaps [nl.SEG6_LOCAL_MAX]bool
+	flags_end_b6_encaps[nl.SEG6_LOCAL_ACTION] = true
+	flags_end_b6_encaps[nl.SEG6_LOCAL_SRH] = true
+	e1 := &SEG6LocalEncap{
+		Flags:    flags_end_b6_encaps,
+		Action:   nl.SEG6_LOCAL_ACTION_END_B6,
+		Segments: s1,
+	}
+	route1 := Route{LinkIndex: link.Attrs().Index, Dst: dst1, Encap: e1}
+
+	// Add SEG6Local routes
+	if err := RouteAdd(&route1); err != nil {
+		t.Fatal(err)
+	}
+
+	// typically one route (fe80::/64) will be created when dummy_route6 is created.
+	// Thus you cannot use RouteList() to find the route entry just added.
+	// Lookup route and confirm it's SEG6Local route just added.
+	routesFound, err := RouteGet(dst1.IP)
+	if err != nil {
+		t.Fatal(err)
+	}
+	if len(routesFound) != 1 { // should only find 1 route entry
+		t.Fatal("SEG6Local route not added correctly")
+	}
+	if !e1.Equal(routesFound[0].Encap) {
+		t.Fatal("Encap does not match the original SEG6LocalEncap")
+	}
+
+	// Del SEG6Local routes
+	if err := RouteDel(&route1); err != nil {
+		t.Fatal(err)
+	}
+	// Confirm route is deleted.
+	routesFound, err = RouteGet(dst1.IP)
+	if err == nil {
+		t.Fatal("SEG6Local route still exists.")
+	}
+
+	// cleanup dummy interface created for the test
+	if err := LinkDel(link); err != nil {
+		t.Fatal(err)
+	}
+}
+
 func TestMTURouteAddDel(t *testing.T) {
 	_, err := RouteList(nil, FAMILY_V4)
 	if err != nil {
